from sqlalchemy.exc import IntegrityError
from werkzeug.exceptions import BadRequest, NotFound

from flaschengeist.models.user import Role, Permission
from flaschengeist.database import db
from flaschengeist import logger
from flaschengeist.utils.hook import Hook


def get_all():
    return Role.query.all()


def get(role_name):
    if type(role_name) is int:
        role = Role.query.get(role_name)
    else:
        role = Role.query.filter(Role.name == role_name).one_or_none()
    if not role:
        raise NotFound
    return role


def get_permissions():
    return Permission.query.all()


@Hook
def role_updated(role, old_name):
    """Hook used when roles are updated"""
    pass


def rename(role, new_name):
    if role.name == new_name:
        return
    if db.session.query(db.exists().where(Role.name == new_name)).scalar():
        raise BadRequest("Name already used")

    old_name = role.name
    role.name = new_name
    role_updated(role, old_name)
    db.session.commit()


def set_permissions(role, permissions):
    role.permissions.clear()
    for name in permissions:
        p = Permission.query.filter(Permission.name.in_(permissions)).all()
        if not p or len(p) < len(permissions):
            raise BadRequest("Invalid permission name >{}<".format(name))
        role.permissions.extend(p)
    db.session.commit()


def create_permissions(permissions):
    for permission in permissions:
        if Permission.query.filter(Permission.name == permission).count() > 0:
            continue
        p = Permission(name=permission)
        db.session.add(p)
        db.session.commit()


def create_role(name: str, permissions=[]):
    role = Role(name=name)
    db.session.add(role)
    set_permissions(role, permissions)
    return role


def delete(role):
    role.permissions.clear()
    try:
        db.session.delete(role)
        db.session.commit()
    except IntegrityError:
        logger.debug("IntegrityError: Role might still be in use", exc_info=True)
        raise BadRequest("Role still in use")
    role_updated(None, role.name)
